library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.my_package.all;
entity control is
port (
clk, reset,start: in std_logic;
first_step, oper_a, msb_a, ce_a, ce_b, ce_u, ce_v, ce_d, ce_p: out std_logic;
cont_u: out std_logic_vector(1 downto 0);
cont_d: out std_logic;
p_gt_0, d_equ_0, sign_d, a_0, u_0, u_plus_v: in std_logic;
done: out std_logic
);
end control;


architecture rtl of control is

signal current_state: std_logic_vector(1 downto 0);
signal first_use, set_first, clear_first: std_logic;

begin

--first_use flag
process(set_first, clear_first)
begin
if set_first = '1' then first_use <= '1';
elsif clear_first = '1' then first_use <= '0';
end if;
end process;

--state transitions
process(clk, reset)
begin
if reset = '1' then current_state <= "00"; 
elsif clk'event and clk = '1' then 
	case current_state is
	when "00" => if start = '0' then current_state <= current_state + 1; end if;
	when "01" => if start = '1' then current_state <= current_state + 1; end if;
	when "10" => current_state <= current_state + 1; 
	when others => if p_gt_0 = '0' then current_state <= current_state + 1; end if;
	end case;
end if;
end process;

--control signals
process(current_state, p_gt_0, d_equ_0, sign_d, a_0, u_0, u_plus_v)
begin
case current_state is

when "00" | "01" =>
first_step <= '0'; oper_a <= '0'; msb_a <= '0'; 
ce_a <= '0'; ce_b <= '0'; ce_u <= '0'; ce_v <= '0'; ce_d <= '0'; ce_p <= '0';
cont_u <= "00"; cont_d <= '0';
done <= '1'; set_first <= '0'; clear_first <= '0';

when "10" =>
first_step <= '1'; oper_a <= '0'; msb_a <= '0'; 
ce_a <= '1'; ce_b <= '1'; ce_u <= '1'; ce_v <= '1'; ce_d <= '1'; ce_p <= '1';
cont_u <= "00"; cont_d <= '0';
done <= '1'; set_first <= '1'; clear_first <= '0';

when others =>
if p_gt_0 = '0' then
	first_step <= '0'; oper_a <= '0'; msb_a <= '0'; 
	ce_a <= '0'; ce_b <= '0'; ce_u <= '0'; ce_v <= '0'; ce_d <= '0'; ce_p <= '0';
	cont_u <= "00"; cont_d <= '0';
	done <= '0'; set_first <= '0'; clear_first <= '0';
else
	if a_0 = '0' then
		first_step <= '0'; oper_a <= '0'; msb_a <= '0'; 
		ce_a <= '1'; ce_b <= '0'; ce_u <= '1'; ce_v <= '0'; ce_d <= '1'; 
		cont_d <= '0'; done <= '0';
		if u_0 = '0' then cont_u <= "00"; else cont_u <= "10"; end if;
		if (sign_d = '1') or (d_equ_0 = '1') then 	ce_p <= '1'; 
		else ce_p <= '0'; end if;
		set_first <= '0'; clear_first <= '0';
	
	else
		first_step <= '0'; oper_a <= '1';  
		ce_a <= '1';  ce_u <= '1'; ce_d <= '1';  
		done <= '0';
		if first_use = '1' then msb_a <= '1'; else msb_a <= '0'; end if;
		if u_plus_v = '0' then cont_u <= "01"; else cont_u <= "11"; end if;
		if (sign_d = '0') then
			ce_b <= '0'; ce_v <= '0';
			if d_equ_0 = '1' then ce_p <= '1'; else ce_p <= '0'; end if;
			cont_d <= '0';
			set_first <= '0'; clear_first <= '0';
		else
			ce_b <= '1'; ce_v <= '1'; ce_p <= '0';
			cont_d <= '1';
			set_first <= '0'; clear_first <= '1';
		end if;
	end if;
end if;
end case;	
end process;
end rtl;

