library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.my_package.all;
entity data_path is
port (
num, den: in std_logic_vector(n-1 downto 0);
clk, reset: in std_logic; 
first_step, oper_a, msb_a, ce_a, ce_b, ce_u, ce_v, ce_d, ce_p: in std_logic;
cont_u: in std_logic_vector(1 downto 0);
cont_d: in std_logic;
z: out std_logic_vector(n-1 downto 0);
p_gt_0, d_equ_0, sign_d, a_0, u_0, u_plus_v: out std_logic
);
end data_path;

architecture circuit of data_path is

component UV_cell
port (
u, v, num: in STD_LOGIC_VECTOR (N-1 downto 0);
cont_u: in std_logic_vector(1 downto 0);
first_step: in std_logic;
new_u, new_v: out STD_LOGIC_VECTOR (N-1 downto 0)
);
end component;

component AB_cell
port (
a, b, den: in STD_LOGIC_VECTOR (N-1 downto 0);
oper_a, msb_a, first_step: in std_logic;
new_a, new_b: out STD_LOGIC_VECTOR (N-1 downto 0)
);
end component;

signal a, b, u, v, new_a, new_b, new_u, new_v: std_logic_vector(n-1 downto 0);
signal d, new_d, p, new_p, p_minus_1, d_minus_1: std_logic_vector(logN downto 0);
signal boolean_sum_p: std_logic_vector(logN-1 downto 0);
signal boolean_sum_d: std_logic_vector(logN-2 downto 0);

begin

uv_block: uv_cell port map (u, v, num, cont_u, first_step, new_u, new_v);

ab_block: ab_cell port map (a, b, den, oper_a, msb_a, first_step, new_a, new_b);

p_minus_1 <= p-1; d_minus_1 <= d-1; 

with first_step select new_p <= n_minus_1 when '1', p_minus_1 when others;

d_counter: for i in 0 to logN generate
new_d(i) <= first_step or (not(cont_d) and d_minus_1(i))or (cont_d and not(d(i)));
end generate;

boolean_sum_p(0) <= p(0);
or_gates_p: for i in 1 to logN-1 generate
boolean_sum_p(i) <= boolean_sum_p(i-1) or p(i);
end generate;

boolean_sum_d(0) <= d(1);
or_gates_d: for i in 1 to logN-2 generate
boolean_sum_d(i) <= boolean_sum_d(i-1) or d(i+1);
end generate;

p_gt_0 <= (not(p(logN))) and boolean_sum_p(logN - 1);
d_equ_0 <= (not(d(logN))) and (not(boolean_sum_d(logN-2))) and (not(d(0)));
sign_d <= d(logN);
a_0 <= a(0);
u_0 <= u(0);
u_plus_v <= u(0) xor v(0);

with a(0) select z <= v when '0', u when others;

process(clk, reset)
begin
if reset = '1' then a <= (others => '0');
elsif clk'event and clk = '1' then
if ce_a = '1' then a <= new_a; end if;
end if;
end process;

process(clk, reset)
begin
if reset = '1' then b <= (others => '0');
elsif clk'event and clk = '1' then
if ce_b = '1' then b <= new_b; end if;
end if;
end process;

process(clk, reset)
begin
if reset = '1' then u <= (others => '0');
elsif clk'event and clk = '1' then
if ce_u = '1' then u <= new_u; end if;
end if;
end process;

process(clk, reset)
begin
if reset = '1' then v <= (others => '0');
elsif clk'event and clk = '1' then
if ce_v = '1' then v <= new_v; end if;
end if;
end process;

process(clk, reset)
begin
if reset = '1' then p <= (others => '0');
elsif clk'event and clk = '1' then
if ce_p = '1' then p <= new_p; end if;
end if;
end process;

process(clk, reset)
begin
if reset = '1' then d <= (others => '0');
elsif clk'event and clk = '1' then
if ce_d = '1' then d <= new_d; end if;
end if;
end process;

end circuit;

